//**********************************************************************
//  DIVE Laboratories, Inc.
//  Copyright(c) 1995
//  All rights reserved.
//  FILE:   GEOMETRY.HPP
//
//  DESCRIPTION
//  This file provides the function prototypes for the geometryClass
//
//  Author: M. Doucet
//
//  Modification History:
//  6/13/95	Created
//
//**********************************************************************
#ifndef _GEOMETRY_HPP
#define _GEOMETRY_HPP

#include "quat.hpp"
#include "vector.hpp"
#include "primset.hpp"
#include "matrix.hpp"


#ifndef AMBERRT
#include "material.hpp"
#endif

#ifndef SHAREWARE
class scsClass;  
#endif
class geoStateClass;
class universeClass;
class geometryPrivateClass;

class geometryClass {

#ifndef SHAREWARE
	friend class scsClass;
#endif
	friend class universeClass;
	friend class lightPrivateClass;
	friend class universePrivateClass;
	friend class geometryPrivateClass;

private:

	geometryPrivateClass *data;

protected:

	sLinkClass *getVertexList();

public:

	// Geometry Sets
	geoPrimSetClass	 lineSet;
	geoPrimSetClass	 polySet;
	geoPrimSetClass	 pointSet;

	// Level Of Detail Functions
	//
	void addLOD(geometryClass *g, Vres range);

	// Hierarchy functions
	//
	void addChild(geometryClass *child);
	void removeChild(geometryClass *child);
	void removeAllChildren();

	// Geometry construction
	//
	int          addVertex(vertexClass *v);
	vertexClass *getVertex(int index);
	void         getVertexPos(int index, V3 pos);

	// Add new primitives whose vertices ARE currently
	// part of the geometry.  If you want the vertices
	// added, set "hasNewVertices" to TRUE.
	void addPoint(pointClass *p);
	void addLine(lineClass *l, int hasNewVertices=FALSE);
	void addPolygon(polygonClass *p, int hasNewVertices=FALSE);

	// Simple primitive retrieval
	//
	pointClass 		*getPoint(int index);
	lineClass 		*getLine(int index);
	polygonClass 	*getPolygon(int index);

	// Visibility
	void setVisibility(int flag);
	int  getVisibility();

	// Extents (local and hierarchy)
	void getExtents(V3 ext);
	void getHierarchyExtents(V3 ext);
	void getHierarchyBounds(V3 pos, V3 ext);

	// Scaling
	void stretch(V3 vals);
	void scale(Vres s);
	void getScaling(V3 vals);
	void scaleHierarchy(Vres s);

	// Position Functions
	//
	void getPosition(V3 pos);
	void setPosition(V3 pos);
	void setPosition(positionClass p);
	void getPosition(positionClass &p);
	void translate(vectorClass delta, frameEnum frame=_LOCAL);
	void translate(vectorClass *delta, frameEnum frame=_LOCAL);

	// Rotation pivot points in world coordinates
	//
	void getPivot(V3 pos);
	void setPivot(V3 pos);
	void setPivot(pivotXenum Xpivot = _XCENTER,
	              pivotYenum Ypivot = _YCENTER,
	              pivotZenum Zpivot = _ZCENTER);

 	// Rotations
	//
	void rotate(int axis, Vres rads);
	void rotate(vectorClass *axis, Vres rads);

	// Orientation Functions
	//
	void initOrientation();
	void setOrientation(quatClass q);
	void getOrientation(quatClass &q);

 	// Alignment
	//
 	void alignAxis(int axis, vectorClass *v);
	void getAxis(int axis, V3 vals);
	void getAxis(int axis, vectorClass &vec);

	// Creating a bounding box
	//
	void           createBoundingBox();
	void           setBoundingBoxVisibility(int flag);
	geometryClass *getBoundingBox();

	// Miscellaneous functions
	//
	int    getNumPolygonsInHierarchy();
	int    getNumPolygons();
	int	 getNumLines();
	int    getNumPoints();
	int    getNumVertices();
	Vres 	 getRadius();
	void   setNormalOverride(int flag);
	void   updateNormals();
	int    getId();
	void   setData(void *pt);
	void  *getData();
	void   localToWorldPos(V3 in, V3 out);
	void   worldToLocalPos(V3 in, V3 out);
	void   complete();
	int    isComplete();

	// Picking Functions
	int rayPickPoly(V3 from, V3 to);

	// Picking Functions
	flagType rayPickGeo(V3 from, V3 to);

	// Local action functions
	//
	void addAction (ACTIONFUNCPTR func, int rate=1, int phase=1);
	void removeAction(ACTIONFUNCPTR func);

	void  setName(char *nm);
	char *getName();
	char *getFilename();

	void setAllColor(F4 col);
	void getAllColor(F4 col);

#ifndef AMBERRT
	void setTexture(textureClass *t, int mode);
	void setMaterial(materialClass *m);
	materialClass *getMaterial();
#endif

	// Adding geometry to universe
	//
	void addTo(universeClass *univ);

	// File Saving
	//
	void save(char *filename, fileEnumType fileType=_NFF_FILE);

	// Collistion state
	//
	void setCollisionState(int flag, int dynamic=FALSE);
	int  getCollisionState();

	// Geometry Combining
	void operator+=(geometryClass *g);
	void add(geometryClass *g);
	void combineHierarchy();

	// Render functions
	void        		enableState(int state);
	void        		disableState(int state);
	int         		getState();

	void 			   	setPolyRenderAs(polyAsEnum as);
	polyAsEnum 	   	getPolyRenderAs();

	void 			   	setPolyFill(polyFillEnum fill);
	polyFillEnum   	getPolyFill();

	void 		   	 	setPolyShading(polyShadingEnum shading);
	polyShadingEnum 	getPolyShading();

	// Constructors & Destructors
	//
	geometryClass(char *filename, 
					  int createBBox=TRUE, int collisionOn=FALSE);
	geometryClass();
	~geometryClass();

};

#endif
